package com.xkrs.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.common.tool.TokenUtil;
import com.xkrs.dao.ForestRangerDao;
import com.xkrs.model.entity.ForestRanger;
import com.xkrs.service.ForestRangerService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

/**
 * @author xkrs
 */
@Service
public class ForestRangerServiceImpl implements ForestRangerService {

    @Resource
    private ForestRangerDao forestRangerDao;

    /**
     * 根据曲线编码查询护林员的信息
     * @param rangerCountyCode
     * @return
     */
    @Override
    public String findByRangerCountyCode(String rangerCountyCode) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        ForestRanger byRangerCountyCode = forestRangerDao.findByRangerCountyCode(rangerCountyCode);
        if(byRangerCountyCode == null){
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"该区县暂时还没有相关护林员的信息",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,byRangerCountyCode,locale);
    }

    /**
     * 根据区县编码查询该区县所有护林员的信息
     * @param countyCode
     * @return
     */
    @Override
    public String findAllByRangerCountyCode(String countyCode, String token) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        String tokenUserName = TokenUtil.getTokenUserName(token);
        if("370000".equals(countyCode)){
            List<ForestRanger> all = forestRangerDao.findAll();
            if(all == null || all.size() == 0){
                return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时还没该省护林员的信息",locale);
            }
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,all,locale);
        }else {
            List<ForestRanger> allByRangerCountyCode = forestRangerDao.findAllsByRangerCountyCode(countyCode);
            if(allByRangerCountyCode == null || allByRangerCountyCode.size() == 0){
                return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时还没该市护林员的信息",locale);
            }
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,allByRangerCountyCode,locale);
        }
    }

    /**
     * 根据手机号修改护林员的位置信息
     * @param rangerLatitude
     * @param rangerLongitude
     * @param rangerPhone
     * @return
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String updateLatAndLonByPhone(String rangerLatitude, String rangerLongitude, String rangerPhone) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        forestRangerDao.updateLatAndLonByPhone(rangerLatitude,rangerLongitude,rangerPhone);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"操作成功",locale);
    }
}
